/* Copyright (C) Gabriel T. Delarosa, 2010. 
 * All rights reserved worldwide.
 *
 * This software is provided "as is" without express or implied
 * warranties. You may freely copy and compile this source into
 * applications you distribute provided that the authors name is
 * mentioned.
 */
#ifndef _MATH_H
#define _MATH_H

#include <d3dx9math.h>

struct Airplane
{
    Airplane():
        yaw_( 0.0f ),
        pitch_( 0.0f ),
        roll_( 0.0f ),
        climbRate_( 0.0f ),
		speed_( 0.0f )
    {
    }

    float yaw_, pitch_, roll_;
    float climbRate_;

    float speed_;

};

namespace vi
{

inline const D3DXQUATERNION operator *( const D3DXQUATERNION &q, const D3DXVECTOR3 &v )
{
    return D3DXQUATERNION(   q.w * v.x + q.y * v.z - q.z * v.y,
                             q.w * v.y + q.z * v.x - q.x * v.z,
                             q.w * v.z + q.x * v.y - q.y * v.x,
                          -( q.x * v.x + q.y * v.y + q.z * v.z )
                         );
}
inline void transformVectorWithQuat( D3DXQUATERNION q, D3DXVECTOR3 &v )
{
    D3DXQUATERNION t = q * v;
    D3DXQuaternionConjugate( &q, &q );
    t = q * t;

    v.x = t.x;
    v.y = t.y;
    v.z = t.z;
}

inline const D3DXVECTOR3 operator *( const D3DXVECTOR3 &u, const D3DXMATRIX &m )
{
    return D3DXVECTOR3( u.x*m(0,0)+ u.y*m(1,0) + u.z*m(2,0) + m(3,0),
                        u.x*m(0,1) + u.y*m(1,1) + u.z*m(2,1) + m(3,1),
                        u.x*m(0,2) + u.y*m(1,2) + u.z*m(2,2) + m(3,2)
                   );
}



}// end namespace

#endif