/* Copyright (C) Gabriel T. Delarosa, 2010. 
 * All rights reserved worldwide.
 *
 * This software is provided "as is" without express or implied
 * warranties. You may freely copy and compile this source into
 * applications you distribute provided that the authors name is
 * mentioned.
 */
#include "OrthographicRenderer.h"

namespace vi
{

HRESULT OrthographicRenderer::init( const ViewPort &vp )
{
    assert( d3dDevice_ );

    D3DXMatrixOrthoRH (&projMatrix_, vp.screenWidth(), vp.screenHeight(), 0.0f, 1.0f);//viewport.viewDistance() );
    d3dDevice_->SetTransform( D3DTS_PROJECTION, &projMatrix_ );

    return S_OK;
}

void OrthographicRenderer::renderCameraFootprint( const std::vector<D3DXVECTOR3> &clippedPts, D3DXVECTOR3 eye, float viewDist )
{
    unsigned int cp = static_cast< unsigned int > ( clippedPts.size() );
    D3DXVECTOR3 footPrintVertex;
    float scale;

    CUSTOMVERTEX *geometryVertices = new CUSTOMVERTEX[ cp*sizeof(CUSTOMVERTEX) ];

    for (unsigned int i=0; i < cp; i++)
    {
        D3DXVec3Normalize( &footPrintVertex, &clippedPts[i] );

        scale = fabs(footPrintVertex.y == 0 ? 0 : eye.y / footPrintVertex.y);
        footPrintVertex *= scale;
        footPrintVertex.x += eye.x;
        footPrintVertex.y += eye.y;
        footPrintVertex.z += eye.z;
    
        geometryVertices[i].position = footPrintVertex * viewMatrix_;
        geometryVertices[i].position.x = ( (geometryVertices[i].position.x / geometryVertices[i].position.z) * -viewDist );
        geometryVertices[i].position.y = ( (geometryVertices[i].position.y / geometryVertices[i].position.z) * -viewDist );
        geometryVertices[i].position.z = 0.0f;
        geometryVertices[i].color = 0xff005500;
    }

    d3dDevice_->SetFVF( D3DFVF_CUSTOMVERTEX );
    d3dDevice_->DrawPrimitiveUP( D3DPT_TRIANGLEFAN, (cp/3) + (cp % 3) , geometryVertices, sizeof(CUSTOMVERTEX) );
    delete [] geometryVertices;
}

}// end namespace
