/* Copyright (C) Gabriel T. Delarosa, 2010. 
 * All rights reserved worldwide.
 *
 * This software is provided "as is" without express or implied
 * warranties. You may freely copy and compile this source into
 * applications you distribute provided that the authors name is
 * mentioned.
 */

#include "PerspectiveRenderer.h"

namespace vi
{

HRESULT PerspectiveRenderer::init( const ViewPort &vp )
{
    assert( d3dDevice_ );

    D3DXMatrixPerspectiveFovRH( &projMatrix_,
                                (vp.fov() * vp.aspectRatio()) *0.01745f,
                                1.0f / vp.aspectRatio(),
                                vp.nearDistance(),
                                vp.farDistance() 
                              );

    d3dDevice_->SetTransform( D3DTS_PROJECTION, &projMatrix_ );

    if ( FAILED( D3DXCreateTextureFromFile( d3dDevice_, "Data/rock007.bmp", &textures_[LOW_GRND] ) ) )
        return E_FAIL;
    if ( FAILED( D3DXCreateTextureFromFile( d3dDevice_, "Data/rock057.bmp", &textures_[HIGH_GRND] ) ) )
        return E_FAIL;
    if ( FAILED( D3DXCreateTextureFromFile( d3dDevice_, "Data/stone.bmp", &textures_[STONE] ) ) )
        return E_FAIL;

    return S_OK;
}

void PerspectiveRenderer::renderCameraFootprint( const std::vector<D3DXVECTOR3> &clippedPts, D3DXVECTOR3 eye, float viewDist )
{
    unsigned int cp = static_cast< unsigned int > ( clippedPts.size() );
    D3DXVECTOR3 footPrintVertex;
    float scale;

    TEXTRVERTEX *geometryVertices = new TEXTRVERTEX[ cp*sizeof(TEXTRVERTEX) ];

    for (unsigned int i=0; i < cp; i++)
    {
        D3DXVec3Normalize( &footPrintVertex, &clippedPts[i] );

        scale = fabs(footPrintVertex.y == 0 ? 0 : eye.y / footPrintVertex.y);
        footPrintVertex *= scale;
        footPrintVertex.x += eye.x;
        footPrintVertex.y += eye.y;
        footPrintVertex.z += eye.z;
    
        geometryVertices[i].position = footPrintVertex;
        geometryVertices[i].tu = footPrintVertex.x / 200.0f;
        geometryVertices[i].tv = footPrintVertex.z / 200.0f;
        geometryVertices[i].ts = footPrintVertex.x / 15000.0f;
        geometryVertices[i].tt = footPrintVertex.z / 15000.0f;
    }

    d3dDevice_->SetTransform( D3DTS_VIEW, &viewMatrix_ );

    d3dDevice_->SetTexture(0, textures_[LOW_GRND]);
    d3dDevice_->SetTextureStageState( 0, D3DTSS_COLORARG1, D3DTA_TEXTURE );
    d3dDevice_->SetTextureStageState( 0, D3DTSS_COLORARG2, D3DTA_DIFFUSE );
    d3dDevice_->SetTextureStageState( 0, D3DTSS_COLOROP,   D3DTOP_MODULATE );
    d3dDevice_->SetTextureStageState( 0, D3DTSS_TEXCOORDINDEX, 0 ); 
    d3dDevice_->SetSamplerState( 0, D3DSAMP_MINFILTER, D3DTEXF_LINEAR );
    d3dDevice_->SetSamplerState( 0, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR );
    d3dDevice_->SetSamplerState( 0, D3DSAMP_MIPFILTER, D3DTEXF_LINEAR );

    d3dDevice_->SetTexture(1, textures_[HIGH_GRND]);
    d3dDevice_->SetTextureStageState( 1, D3DTSS_COLORARG1, D3DTA_CURRENT );
    d3dDevice_->SetTextureStageState( 1, D3DTSS_COLORARG2, D3DTA_TEXTURE );
    d3dDevice_->SetTextureStageState( 1, D3DTSS_COLOROP,   D3DTOP_MODULATE );
    d3dDevice_->SetTextureStageState( 1, D3DTSS_TEXCOORDINDEX, 1 ); 
    d3dDevice_->SetSamplerState( 1, D3DSAMP_MINFILTER, D3DTEXF_LINEAR );
    d3dDevice_->SetSamplerState( 1, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR );
    d3dDevice_->SetSamplerState( 1, D3DSAMP_MIPFILTER, D3DTEXF_LINEAR );


    d3dDevice_->SetFVF( D3DFVF_TEXTRVERTEX );
    d3dDevice_->DrawPrimitiveUP( D3DPT_TRIANGLEFAN, (cp/3) + (cp % 3), geometryVertices, sizeof(TEXTRVERTEX) );

    delete [] geometryVertices;

}

}// end namespace