/* Copyright (C) Gabriel T. Delarosa, 2010. 
 * All rights reserved worldwide.
 *
 * This software is provided "as is" without express or implied
 * warranties. You may freely copy and compile this source into
 * applications you distribute provided that the authors name is
 * mentioned.
 */

#include "QCamera.h"
#include "Math.h"

namespace vi
{
QCamera::QCamera()
{
    set( 0,10,0, 0,10,-1, 0,1,0 );
}

QCamera::QCamera( float posX,  float posY,  float posZ,
                  float lookAtX, float lookAtY, float lookAtZ,
                  float upX,   float upY,   float upZ )
{
    set( posX, posY, posZ, lookAtX, lookAtY, lookAtZ, upX, upY, upZ );
}

void QCamera::set( float posX,  float posY,  float posZ,
                   float lookAtX, float lookAtY, float lookAtZ,
                   float upX,   float upY,   float upZ )
{
    eye_ = D3DXVECTOR3( posX, posY, posZ );
    lookAt_ = eye_ - D3DXVECTOR3( lookAtX, lookAtY, lookAtZ );

    D3DXVECTOR3 temp = D3DXVECTOR3( upX, upY, upZ );

    D3DXVec3Cross( &side_, &temp, &lookAt_ );
    D3DXVec3Cross( &up_, &lookAt_, &side_ );
    
    D3DXVec3Normalize( &side_, &side_ );
    D3DXVec3Normalize( &up_, &up_ ); 
    D3DXVec3Normalize( &lookAt_, &lookAt_ );

    lookAt_ = -lookAt_;

    D3DXQuaternionRotationYawPitchRoll( &quat_, 0, 0, 0 );
}

void QCamera::translate( float x, float y, float z )
{
    eye_ += side_*x + up_*y + lookAt_*z;
}

void QCamera::yawPitchRoll( float y, float p, float r )
{
    D3DXQUATERNION tempQ;
    D3DXQuaternionRotationYawPitchRoll( &tempQ, y, p, r );
    quat_ *= tempQ;
}

void QCamera::update()
{
    side_   = D3DXVECTOR3(1,0,0);
    up_     = D3DXVECTOR3(0,1,0);
    lookAt_ = D3DXVECTOR3(0,0,-1);
    D3DXQUATERNION conjQ;
    D3DXQuaternionConjugate( &conjQ, &quat_ );

    transformVectorWithQuat( conjQ, side_ );
    transformVectorWithQuat( conjQ, up_ );
    transformVectorWithQuat( conjQ, lookAt_ );
}

}// end namespace