/* Copyright (C) Gabriel T. Delarosa, 2010. 
 * All rights reserved worldwide.
 *
 * This software is provided "as is" without express or implied
 * warranties. You may freely copy and compile this source into
 * applications you distribute provided that the authors name is
 * mentioned.
 */

#ifndef _RENDERER_BASE_H
#define _RENDERER_BASE_H

#include <assert.h>
#include <vector>
#include "Math.h"
#include "ViewPort.h"
#include "QCamera.h"

namespace vi
{

class RendererBase
{
public:
    RendererBase();
    ~RendererBase();

    HRESULT            initD3D( HWND hWnd );
    virtual HRESULT    init( const ViewPort &vp ) = 0;
    void               render( QCamera &cam, const ViewPort &vp, const Airplane &airplane );

    void               drawHorizon( QCamera &qcamera, const ViewPort &viewport );
    virtual void       renderCameraFootprint( const std::vector<D3DXVECTOR3> &clippedPts, D3DXVECTOR3 eye, float viewDist ) = 0;

    FILE *dbf;//temp

protected:
    LPDIRECT3D9          d3d_;
    LPDIRECT3DDEVICE9    d3dDevice_;

    D3DXMATRIX           viewMatrix_;
    D3DXMATRIX           projMatrix_;
};


}// end namespace

#endif