/* Copyright (C) Gabriel T. Delarosa, 2010. 
 * All rights reserved worldwide.
 *
 * This software is provided "as is" without express or implied
 * warranties. You may freely copy and compile this source into
 * applications you distribute provided that the authors name is
 * mentioned.
 */

#ifndef _SCAN_CONVERTER_H
#define _SCAN_CONVERTER_H

#include "Edge.h"
#include <list>
#include <vector>

namespace vi
{

class ScanConverter
{
public:
    typedef std::list<Edge*>            EdgePtrListType;
    typedef EdgePtrListType::iterator   EdgePtrListIterator;
    typedef std::pair<int, int>         CellType;
    typedef std::vector<CellType>       CellVector;

    ScanConverter(): scan_(0), deltaScan_(1){}

public:
    void insertEdge( const D3DXVECTOR2& u, const D3DXVECTOR2& v );
    void rasterize( CellVector &outputCells );
    bool hasEdges()const { return !edges_.empty(); }

private:
    Edge *makeEdge( const D3DXVECTOR2& u, const D3DXVECTOR2& v ) const;
    void adjustRightEdge();
    void removeScannedEdges();

    inline void updateEdge( const EdgePtrListIterator &eIt );
    inline void getFrontTwoEdges( EdgePtrListIterator &head, EdgePtrListIterator &next );

    EdgePtrListType       edges_;
    EdgePtrListIterator   rightEdge_;

    float scan_;
    float deltaScan_;
};

} //end namespace

#endif