/* Copyright (C) Gabriel T. Delarosa, 2010. 
 * All rights reserved worldwide.
 *
 * This software is provided "as is" without express or implied
 * warranties. You may freely copy and compile this source into
 * applications you distribute provided that the authors name is
 * mentioned.
 */
#ifndef _TERRAIN_RENDER_H
#define _TERRAIN_RENDER_H

#include "PerspectiveRenderer.h"


namespace vi
{
const int HMAP_DIM = 128;
const float MIN_CELL_SIZE = 1000.0f;

class TerrainRenderer : public PerspectiveRenderer
{
    struct TERRAINVERTEX
    {
        D3DXVECTOR3 position;
        FLOAT       tu, tv;
    };
    #define D3DFVF_TERRAINVERTEX ( D3DFVF_XYZ | D3DFVF_TEX1 )

public:
    ~TerrainRenderer();
    
    HRESULT init( const ViewPort &vp );
    void  renderCameraFootprint( const std::vector<D3DXVECTOR3> &clippedPts, D3DXVECTOR3 eye, float viewDist );

private:
    void renderCell( TERRAINVERTEX *geometryVertices, int x, int y, float camY, float cellSize );
    void renderTerrainCell( int x, int y, float camY, float cellSize );
    inline float getHeight( int tx, int tz ) { return htmap_[tz][tx] * 0.5f; }

    inline float getScale( int iY );

    unsigned char htmap_[HMAP_DIM][HMAP_DIM];    
    TERRAINVERTEX *terrainVerts_;
};

}// end namespace

#endif