/* Copyright (C) Gabriel T. Delarosa, 2010. 
 * All rights reserved worldwide.
 *
 * This software is provided "as is" without express or implied
 * warranties. You may freely copy and compile this source into
 * applications you distribute provided that the authors name is
 * mentioned.
 */

#ifndef _VIEWPORT_H
#define _VIEWPORT_H

#include <d3dx9math.h>

namespace vi
{

class ViewPort
{
public:
    ViewPort();
    ViewPort( float fov, float scrW, float scrH, float nearD, float farD );

    void set( float fov, float scrW, float scrH, float nearD, float farD );

    void setFOV( float fov ) 
    { 
        fov_ = fov;
        setViewDistance();
    }

    void setScreenDims( float scrW, float scrH )
    { 
        screenWidth_ = scrW;
        screenHeight_ = scrH;
        setViewDistance();
        setAspectRatio();
    }

    void setNearDistance( float d ) { near_ = d; }
    void setFarDistance( float d ) { far_ = d; }

    float fov() const { return fov_; }
    float screenWidth() const { return screenWidth_; }
    float screenHeight() const { return screenHeight_; }
    float aspectRatio() const { return aspectRatio_; }
    float nearDistance() const { return near_; }
    float farDistance() const { return far_; }
    float viewDistance() const { return viewDistance_; }

private:
    void setViewDistance() const
    {
        viewDistance_ = ( (screenWidth_*0.5f) / tan((fov_*0.5f)*.01745f) );
    }
    void setAspectRatio() const
    {
        aspectRatio_ = screenHeight_ / screenWidth_;
    }

private:
    float          fov_;
    float          screenWidth_;
    float          screenHeight_;
    float          near_;
    float          far_;
    mutable float  viewDistance_;
    mutable float  aspectRatio_;
};


}// end namespace

#endif